using System;
using System.Windows.Forms; // for Application

namespace MyNamespace
{
	/// <summary>
	///		Startup form for JosephusCS program
	///		Developed in TURBO Pascal 5.5 and 6.0, converted to Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class StartupForm : Form
	{
		private Button buttonDefault;
		private Button buttonExit;
		private Button buttonRun;
		private Label labelFirstF;
		private Label labelMaxN;
		private Label labelMaxP;
		private Label labelNumberN;
		private Label labelSkipM;
		private RichTextBox richTextIntro;
		private System.ComponentModel.IContainer components;
		private TextBox textBoxFirstF;
		private TextBox textBoxMaxN;
		private TextBox textBoxMaxP;
		private TextBox textBoxNumberN;
		private TextBox textBoxSkipM;
		private Timer timerStartup;

		public StartupForm()
		{
			InitializeComponent();
			maxMaxN = 1000000;		// [1, max]
			maxNumberN = 1000000;	// [1, max]
			maxFirstF = 1000000;	// [1, max]
			maxSkipM = 1000000;		// [1, max]
			maxMaxP = 1000000;		// [1, max]
			defaultMaxN = 1000;
			defaultNumberN = 100;
			defaultFirstF = 1;
			defaultSkipM = 2;
			defaultMaxP = 200;
			SetDefault();
			this.Text = Global.title + " - Startup";
			Pr("");
			Pr("   " + Global.name1);
			Pr("   " + Global.name2);
			Pr("   " + Global.version);
			Pr("   " + Global.author);
			Pr("   " + Global.address);
			Pr("");
			Pr("JosephusCS is a program that solves the Josephus Permutation problem:");
			Pr("");
			Pr("Original Problem: 1000 slaves in a circle, numbered 1 to 1000, are all");
			Pr("to be shot except one lucky survivor. The order of shooting is 1, 3,");
			Pr("5, etc., always alternating, and always immediately removing the");
			Pr("fallen bodies. Once a body has fallen, it is no longer considered part");
			Pr("of the circle for purposes of future counting and alternation.");
			Pr("Example, n=6: Shoot 1,3,5,2,6; 4 survives.");
			Pr("");
			Pr("The General Problem: There is an ordered set of n objects arranged");
			Pr("in a circle with object i (1 <= i <= n) in position i. All n objects are");
			Pr("selected and removed in a certain order and placed in a new circle");
			Pr("with the new position number k beings the order of selection. Object");
			Pr("f is selected first. After each selection, m minus 1 of the remaining");
			Pr("objects following the one removed are skipped and the next object");
			Pr("is then selected. We are interested in the nature of the permutation");
			Pr("generated by this process, its fixed elements, and in particular the");
			Pr("original position L of the last object selected. Note that m and f can");
			Pr("be as low as 1 and can be larger than n.");
			Pr("");
			Pr("n = The number of objects in the circle");
			Pr("f = index of first object to be selected [1, n]");
			Pr("m = difference in index of items selected, m = 2 => every other");
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(StartupForm));
			this.richTextIntro = new RichTextBox();
			this.buttonRun = new Button();
			this.buttonExit = new Button();
			this.labelMaxN = new Label();
			this.textBoxMaxN = new TextBox();
			this.buttonDefault = new Button();
			this.timerStartup = new Timer(this.components);
			this.labelNumberN = new Label();
			this.textBoxNumberN = new TextBox();
			this.labelFirstF = new Label();
			this.textBoxFirstF = new TextBox();
			this.labelSkipM = new Label();
			this.textBoxSkipM = new TextBox();
			this.labelMaxP = new Label();
			this.textBoxMaxP = new TextBox();
			this.SuspendLayout();
			// 
			// richTextIntro
			// 
			this.richTextIntro.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(208)))), ((int)(((byte)(244)))), ((int)(((byte)(255)))));
			this.richTextIntro.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.richTextIntro.ForeColor = System.Drawing.Color.Blue;
			this.richTextIntro.Location = new System.Drawing.Point(0, -8);
			this.richTextIntro.Name = "richTextIntro";
			this.richTextIntro.ReadOnly = true;
			this.richTextIntro.ScrollBars = RichTextBoxScrollBars.None;
			this.richTextIntro.Size = new System.Drawing.Size(464, 513);
			this.richTextIntro.TabIndex = 7;
			this.richTextIntro.Text = "";
			this.richTextIntro.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// buttonRun
			// 
			this.buttonRun.Cursor = Cursors.Arrow;
			this.buttonRun.Location = new System.Drawing.Point(480, 432);
			this.buttonRun.Name = "buttonRun";
			this.buttonRun.Size = new System.Drawing.Size(40, 24);
			this.buttonRun.TabIndex = 0;
			this.buttonRun.Text = "Run";
			this.buttonRun.Click += new System.EventHandler(this.buttonRun_Click);
			this.buttonRun.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// buttonExit
			// 
			this.buttonExit.Cursor = Cursors.Arrow;
			this.buttonExit.Location = new System.Drawing.Point(536, 432);
			this.buttonExit.Name = "buttonExit";
			this.buttonExit.Size = new System.Drawing.Size(40, 23);
			this.buttonExit.TabIndex = 1;
			this.buttonExit.Text = "Exit";
			this.buttonExit.Click += new System.EventHandler(this.buttonExit_Click);
			this.buttonExit.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// labelMaxN
			// 
			this.labelMaxN.Location = new System.Drawing.Point(501, 43);
			this.labelMaxN.Name = "labelMaxN";
			this.labelMaxN.Size = new System.Drawing.Size(81, 28);
			this.labelMaxN.TabIndex = 3;
			this.labelMaxN.Text = "Max Number n <= 1000000";
			// 
			// textBoxMaxN
			// 
			this.textBoxMaxN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.textBoxMaxN.Location = new System.Drawing.Point(504, 74);
			this.textBoxMaxN.Name = "textBoxMaxN";
			this.textBoxMaxN.Size = new System.Drawing.Size(72, 20);
			this.textBoxMaxN.TabIndex = 5;
			this.textBoxMaxN.Text = "16000";
			this.textBoxMaxN.TextChanged += new System.EventHandler(this.textBoxMaxN_TextChanged);
			this.textBoxMaxN.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// buttonDefault
			// 
			this.buttonDefault.Location = new System.Drawing.Point(480, 373);
			this.buttonDefault.Name = "buttonDefault";
			this.buttonDefault.Size = new System.Drawing.Size(104, 24);
			this.buttonDefault.TabIndex = 6;
			this.buttonDefault.Text = "Default Settings";
			this.buttonDefault.Click += new System.EventHandler(this.buttonDefault_Click);
			this.buttonDefault.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// timerStartup
			// 
			this.timerStartup.Enabled = true;
			this.timerStartup.Tick += new System.EventHandler(this.timerStartup_Tick);
			// 
			// labelNumberN
			// 
			this.labelNumberN.Location = new System.Drawing.Point(512, 120);
			this.labelNumberN.Name = "labelNumberN";
			this.labelNumberN.Size = new System.Drawing.Size(70, 16);
			this.labelNumberN.TabIndex = 8;
			this.labelNumberN.Text = "Number n";
			// 
			// textBoxNumberN
			// 
			this.textBoxNumberN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.textBoxNumberN.Location = new System.Drawing.Point(504, 139);
			this.textBoxNumberN.Name = "textBoxNumberN";
			this.textBoxNumberN.Size = new System.Drawing.Size(72, 20);
			this.textBoxNumberN.TabIndex = 9;
			this.textBoxNumberN.Text = "100";
			this.textBoxNumberN.TextChanged += new System.EventHandler(this.textBoxNumberN_TextChanged);
			this.textBoxNumberN.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// labelFirstF
			// 
			this.labelFirstF.Location = new System.Drawing.Point(523, 180);
			this.labelFirstF.Name = "labelFirstF";
			this.labelFirstF.Size = new System.Drawing.Size(53, 16);
			this.labelFirstF.TabIndex = 10;
			this.labelFirstF.Text = "First f";
			// 
			// textBoxFirstF
			// 
			this.textBoxFirstF.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.textBoxFirstF.Location = new System.Drawing.Point(504, 199);
			this.textBoxFirstF.Name = "textBoxFirstF";
			this.textBoxFirstF.Size = new System.Drawing.Size(72, 20);
			this.textBoxFirstF.TabIndex = 11;
			this.textBoxFirstF.Text = "1";
			this.textBoxFirstF.TextChanged += new System.EventHandler(this.textBoxFirstF_TextChanged);
			this.textBoxFirstF.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// labelSkipM
			// 
			this.labelSkipM.Location = new System.Drawing.Point(518, 240);
			this.labelSkipM.Name = "labelSkipM";
			this.labelSkipM.Size = new System.Drawing.Size(64, 16);
			this.labelSkipM.TabIndex = 12;
			this.labelSkipM.Text = "Skip m";
			// 
			// textBoxSkipM
			// 
			this.textBoxSkipM.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.textBoxSkipM.Location = new System.Drawing.Point(504, 259);
			this.textBoxSkipM.Name = "textBoxSkipM";
			this.textBoxSkipM.Size = new System.Drawing.Size(72, 20);
			this.textBoxSkipM.TabIndex = 13;
			this.textBoxSkipM.Text = "2";
			this.textBoxSkipM.TextChanged += new System.EventHandler(this.textBoxSkipM_TextChanged);
			this.textBoxSkipM.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// labelMaxP
			// 
			this.labelMaxP.Location = new System.Drawing.Point(490, 307);
			this.labelMaxP.Name = "labelMaxP";
			this.labelMaxP.Size = new System.Drawing.Size(108, 16);
			this.labelMaxP.TabIndex = 14;
			this.labelMaxP.Text = "Max Points Plotted";
			// 
			// textBoxMaxP
			// 
			this.textBoxMaxP.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.textBoxMaxP.Location = new System.Drawing.Point(504, 326);
			this.textBoxMaxP.Name = "textBoxMaxP";
			this.textBoxMaxP.Size = new System.Drawing.Size(72, 20);
			this.textBoxMaxP.TabIndex = 15;
			this.textBoxMaxP.Text = "200";
			this.textBoxMaxP.TextChanged += new System.EventHandler(this.textBoxMaxP_TextChanged);
			this.textBoxMaxP.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			// 
			// StartupForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Control;
			this.ClientSize = new System.Drawing.Size(600, 504);
			this.Controls.Add(this.textBoxMaxP);
			this.Controls.Add(this.labelMaxP);
			this.Controls.Add(this.textBoxSkipM);
			this.Controls.Add(this.labelSkipM);
			this.Controls.Add(this.textBoxFirstF);
			this.Controls.Add(this.labelFirstF);
			this.Controls.Add(this.textBoxNumberN);
			this.Controls.Add(this.labelNumberN);
			this.Controls.Add(this.buttonDefault);
			this.Controls.Add(this.textBoxMaxN);
			this.Controls.Add(this.labelMaxN);
			this.Controls.Add(this.buttonExit);
			this.Controls.Add(this.buttonRun);
			this.Controls.Add(this.richTextIntro);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "StartupForm";
			this.StartPosition = FormStartPosition.CenterScreen;
			this.Text = "<App.Title> - Startup";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.memberFormClosing);
			this.KeyDown += new KeyEventHandler(this.buttonRun_KeyDown);
			this.ResumeLayout(false);
			this.PerformLayout();

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) // get command line arguments and bring up the frame
		{
			new Global();	// initialize class Global
			Global.parmsDone = true;	// don't process command line arguments
			StartupForm objStartupForm = new StartupForm();
			startupTicks = 1;
			if (!objStartupForm.IsDisposed) Application.Run(objStartupForm);
		}

		private void Pr(string st)
		{
			richTextIntro.Text += "   " + st + "\n";
		}

		private void textBoxMaxN_TextChanged(object sender, System.EventArgs e)
		// max n in a Working array [1, MaxN]
		{
			int iNum = 0;
			string sNum = textBoxMaxN.Text.TrimStart(' ');	// remove leading spaces
			sNum = sNum.TrimStart('0');						// remove leading zeros
			if (sNum == "")
				sNum = "0";
			textBoxMaxN.Text = sNum;						// from text box
			if (Global.IsInt32(sNum))
			{
				iNum = System.Convert.ToInt32(sNum);
				bMaxN = (1 <= iNum) && (iNum <= maxMaxN);
			}
			else
			{
				bMaxN = false;
			}
			int n = sNum.Length;
			if (n > 0 && sNum[n-1] == ' ') // remove space at end
			{
				sNum = sNum.TrimEnd(' ');
				textBoxMaxN.Text = sNum;
				textBoxMaxN.SelectionStart = sNum.Length; // stay at end
			}
			if (bMaxN)	// check NumberN for new maxN
			{
				maxNumberN = iNum;
				textBoxNumberN.Text = " " + textBoxNumberN.Text;
			}
			SetFonts();
		}

		private void textBoxNumberN_TextChanged(object sender, EventArgs e)
		// Number n
		{
			string sNum = textBoxNumberN.Text.TrimStart(' ');	// remove leading spaces
			sNum = sNum.TrimStart('0');							// remove leading zeros
			if (sNum == "")
				sNum = "0";
			textBoxNumberN.Text = sNum;							// from text box
			if (Global.IsInt32(sNum))
			{
				int iNum = System.Convert.ToInt32(sNum);
				bNumberN = (1 <= iNum) && (iNum <= maxNumberN);
			}
			else
			{
				bNumberN = false;
			}
			int n = sNum.Length;
			if (n > 0 && sNum[n - 1] == ' ') // remove space at end
			{
				sNum = sNum.TrimEnd(' ');
				textBoxNumberN.Text = sNum;
				textBoxNumberN.SelectionStart = sNum.Length; // stay at end
			}
			SetFonts();
		}

		private void textBoxFirstF_TextChanged(object sender, EventArgs e)
		// First f
		{
			string sNum = textBoxFirstF.Text.TrimStart(' ');	// remove leading spaces
			sNum = sNum.TrimStart('0');							// remove leading zeros
			if (sNum == "")
				sNum = "0";
			textBoxFirstF.Text = sNum;							// from text box
			if (Global.IsInt32(sNum))
			{
				int iNum = System.Convert.ToInt32(sNum);
				bFirstF = (1 <= iNum) && (iNum <= maxFirstF);
			}
			else
			{
				bFirstF = false;
			}
			int n = sNum.Length;
			if (n > 0 && sNum[n - 1] == ' ') // remove space at end
			{
				sNum = sNum.TrimEnd(' ');
				textBoxFirstF.Text = sNum;
				textBoxFirstF.SelectionStart = sNum.Length; // stay at end
			}
			SetFonts();
		}

		private void textBoxSkipM_TextChanged(object sender, EventArgs e)
		// Skip m
		{
			string sNum = textBoxSkipM.Text.TrimStart(' ');	// remove leading spaces
			sNum = sNum.TrimStart('0');						// remove leading zeros
			if (sNum == "")
				sNum = "0";
			textBoxSkipM.Text = sNum;						// from text box
			if (Global.IsInt32(sNum))
			{
				int iNum = System.Convert.ToInt32(sNum);
				bSkipM = (1 <= iNum) && (iNum <= maxSkipM);
			}
			else
			{
				bSkipM = false;
			}
			int n = sNum.Length;
			if (n > 0 && sNum[n - 1] == ' ') // remove space at end
			{
				sNum = sNum.TrimEnd(' ');
				textBoxSkipM.Text = sNum;
				textBoxSkipM.SelectionStart = sNum.Length; // stay at end
			}
			SetFonts();
		}

		private void textBoxMaxP_TextChanged(object sender, EventArgs e)
		// max n in a Working array [1, MaxN]
		{
			int iNum = 0;
			string sNum = textBoxMaxP.Text.TrimStart(' ');	// remove leading spaces
			sNum = sNum.TrimStart('0');						// remove leading zeros
			if (sNum == "")
				sNum = "0";
			textBoxMaxP.Text = sNum;						// from text box
			if (Global.IsInt32(sNum))
			{
				iNum = System.Convert.ToInt32(sNum);
				bMaxP = (1 <= iNum) && (iNum <= maxMaxP);
			}
			else
			{
				bMaxP = false;
			}
			int n = sNum.Length;
			if (n > 0 && sNum[n - 1] == ' ') // remove space at end
			{
				sNum = sNum.TrimEnd(' ');
				textBoxMaxP.Text = sNum;
				textBoxMaxP.SelectionStart = sNum.Length; // stay at end
			}
			SetFonts();
		}

		private void buttonDefault_Click(object sender, System.EventArgs e)
		{
			SetDefault();
		}

		private void SetDefault()
		{
			textBoxMaxN.Text = defaultMaxN.ToString();
			textBoxNumberN.Text = defaultNumberN.ToString();
			textBoxFirstF.Text = defaultFirstF.ToString();
			textBoxSkipM.Text = defaultSkipM.ToString();
			textBoxMaxP.Text = defaultMaxP.ToString();
			bMaxN = bNumberN = bFirstF = bSkipM = bMaxP = buttonRun.Enabled = true;
			SetFonts();
		}

		private void buttonRun_Click(object sender, System.EventArgs e)
		// start running the program
		{
			buttonRunProg();
		}

		private void buttonRunProg() // start running the program
		{
			int iMaxN = System.Convert.ToInt32(textBoxMaxN.Text);
			int iNumberN = System.Convert.ToInt32(textBoxNumberN.Text);
			int iFirstF = System.Convert.ToInt32(textBoxFirstF.Text);
			int iSkipM = System.Convert.ToInt32(textBoxSkipM.Text);
			int iMaxP = System.Convert.ToInt32(textBoxMaxP.Text);
			this.Hide();
			objRunForm = new RunForm();
			objRunForm.Show();
			objRunForm.Run(iMaxN, iNumberN, iFirstF, iSkipM, iMaxP, this); // call sub Run in the form frmRun
		}

		private void SetFonts()
		{
			if (bMaxN)
				this.textBoxMaxN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxMaxN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			if (bNumberN)
				this.textBoxNumberN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxNumberN.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			if (bFirstF)
				this.textBoxFirstF.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxFirstF.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			if (bSkipM)
				this.textBoxSkipM.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxSkipM.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			if (bMaxP)
				this.textBoxMaxP.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxMaxP.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			buttonRun.Enabled = bMaxN && bNumberN && bFirstF && bSkipM && bMaxP;
		}

		public void TotalQuit()
		{
			Dispose(true);
		}

		private void buttonExit_Click(object sender, System.EventArgs e)
		{
			Dispose(true);
		}

		private void buttonRun_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Escape)		// ESC key pressed so Exit Startup
			{
				Dispose(true);
			}
		}

		private void timerStartup_Tick(object sender, System.EventArgs e)
		{
			if (startupTicks > 0) startupTicks++;
			if (startupTicks > 100)
				timerStartup.Enabled = false;	// and then only needed once
		}

		private void memberFormClosing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			Dispose(true);
		}

		private static bool bFirstF;			// FirstF good flag
		private static bool bMaxN;				// MaxN good flag
		private static bool bMaxP;				// MaxP good flag
		private static bool bNumberN;			// NumberN good flag
		private static bool bSkipM;				// SkipM good flag
		private static int defaultFirstF;		// FirstF default
		private static int defaultMaxN;			// MaxN default
		private static int defaultMaxP;			// MaxP default
		private static int defaultNumberN;		// NumferN default
		private static int defaultSkipM;		// SkipM default
		private static int maxFirstF;			// Max FirstF allowed
		private static int maxMaxN;				// Max MaxN allowed
		private static int maxMaxP;				// Max MaxP allowed
		private static int maxNumberN;			// Max NumberN allowed
		private static int maxSkipM;			// Max SkipM allowed
		private static int startupTicks = 0;
		public static RunForm objRunForm;
	}
}
